#include <clutter/clutter.h>

static void
phase_3_cb (ClutterAnimation *animation,
            ClutterActor     *actor)
{
  clutter_actor_animate (actor, CLUTTER_LINEAR, 750,
                         "x", CLUTTER_STAGE_WIDTH (),
                         "fixed::rotation-angle-x", 0.f,
                         NULL);
}

static void
phase_2_cb (ClutterAnimation *animation,
            ClutterActor     *actor)
{
  ClutterVertex center = { 0, clutter_actor_get_height (actor) / 2.f, 0 };
  g_signal_connect_after (
    clutter_actor_animate (actor, CLUTTER_LINEAR, 500,
                           "rotation-angle-x", 360.f,
                           "fixed::rotation-center-x", &center,
                           "fixed::rotation-angle-y", 0.f,
                           NULL),
    "completed", G_CALLBACK (phase_3_cb), actor);
}

static void
phase_1_cb (ClutterAnimation *animation,
            ClutterActor     *actor)
{
  ClutterVertex center = { clutter_actor_get_width (actor) / 2.f, 0, 0 };
  g_signal_connect_after (
    clutter_actor_animate (actor, CLUTTER_LINEAR, 500,
                           "rotation-angle-y", 360.f,
                           "fixed::rotation-center-y", &center,
                           NULL),
    "completed", G_CALLBACK (phase_2_cb), actor);
}

static gboolean
phase_0_cb (ClutterActor *actor)
{
  /* Setup actor position */
  clutter_actor_set_position (actor,
                              -clutter_actor_get_width (actor),
                              (CLUTTER_STAGE_HEIGHT () -
                               clutter_actor_get_height (actor)) / 2.f);
  clutter_actor_show (actor);

  /* Start animation and attach to the end of it */
  g_signal_connect_after (
    clutter_actor_animate (actor, CLUTTER_LINEAR, 750,
                           "x", (CLUTTER_STAGE_WIDTH () -
                                 clutter_actor_get_width (actor)) / 2,
                           NULL),
    "completed", G_CALLBACK (phase_1_cb), actor);
  return TRUE;
}

int
main (int argc, char **argv)
{
  gfloat width, height;
  ClutterActor *stage, *background, *actor;

  /* Initialise clutter */
  clutter_init (&argc, &argv);

  /* Create a fullscreen stage */
  stage = clutter_stage_get_default ();
  clutter_stage_set_fullscreen (CLUTTER_STAGE (stage), TRUE);
  clutter_actor_get_size (stage, &width, &height);

  /* Add a backgroud at the size of the stage */
  background = clutter_texture_new_from_file ("images/background.jpg", NULL);
  clutter_actor_set_size (background, width, height);
  clutter_container_add_actor (CLUTTER_CONTAINER (stage), background);

  /* Add an actor */
  actor = clutter_texture_new_from_file ("images/actor.png", NULL);
  clutter_actor_set_size (actor, width/5, height/5);
  clutter_actor_hide (actor);
  clutter_container_add_actor (CLUTTER_CONTAINER (stage), actor);

  /* Start animation when we click */
  g_signal_connect_swapped (stage, "button-press-event",
                            G_CALLBACK (phase_0_cb), actor);

  /* Close when we press a key */
  g_signal_connect (stage, "key-press-event",
                    G_CALLBACK (clutter_main_quit), NULL);

  /* Startup the application */
  clutter_actor_show (stage);
  clutter_main ();

  return 0;
}

