#include <clutter/clutter.h>

static gboolean
animate_cb (ClutterActor *actor)
{
  GValue value = {0, };
  ClutterAnimation *anim;
  ClutterInterval *interval;
  ClutterTimeline *timeline;
  ClutterVertex center = { clutter_actor_get_width (actor) / 2.f,
                           clutter_actor_get_height (actor) / 2.f, 0 };

  clutter_actor_set_position (actor,
                              -clutter_actor_get_width (actor),
                              (CLUTTER_STAGE_HEIGHT () -
                               clutter_actor_get_height (actor)) / 2.f);
  g_object_set (actor,
                "rotation-center-x", &center,
                "rotation-center-y", &center,
                NULL);
  clutter_actor_show (actor);

  /* Animation phase 1 */
  anim = clutter_animation_new ();
  clutter_animation_set_object (anim, G_OBJECT (actor));
  clutter_animation_set_duration (anim, 750);
  clutter_animation_set_mode (anim, CLUTTER_EASE_OUT_QUAD);
  g_value_init (&value, G_TYPE_FLOAT);
  g_value_set_float (&value, (CLUTTER_STAGE_WIDTH () -
                              clutter_actor_get_width (actor)) / 2.f);
  clutter_animation_bind (anim, "x", &value);
  timeline = clutter_animation_get_timeline (anim);
  clutter_timeline_start (timeline);
  g_signal_connect (anim, "completed", G_CALLBACK (g_object_unref), NULL);
  g_value_unset (&value);

  /* Animation phase 2 */
  anim = clutter_animation_new ();
  clutter_animation_set_object (anim, G_OBJECT (actor));
  clutter_animation_set_duration (anim, 500);
  clutter_animation_set_mode (anim, CLUTTER_EASE_IN_OUT_QUAD);
  interval = clutter_interval_new (G_TYPE_DOUBLE, 0.0, 360.0);
  clutter_animation_bind_interval (anim, "rotation-angle-y", interval);
  timeline = clutter_animation_get_timeline (anim);
  clutter_timeline_set_delay (timeline, 600);
  clutter_timeline_start (timeline);
  g_signal_connect (anim, "completed", G_CALLBACK (g_object_unref), NULL);

  /* Animation phase 3 */
  anim = clutter_animation_new ();
  clutter_animation_set_object (anim, G_OBJECT (actor));
  clutter_animation_set_duration (anim, 750);
  clutter_animation_set_mode (anim, CLUTTER_EASE_IN_OUT_QUAD);
  interval = clutter_interval_new (G_TYPE_DOUBLE, 0.0, 360.0);
  clutter_animation_bind_interval (anim, "rotation-angle-x", interval);
  timeline = clutter_animation_get_timeline (anim);
  clutter_timeline_set_delay (timeline, 950);
  clutter_timeline_start (timeline);
  g_signal_connect (anim, "completed", G_CALLBACK (g_object_unref), NULL);

  /* Animation phase 4 */
  anim = clutter_animation_new ();
  clutter_animation_set_object (anim, G_OBJECT (actor));
  clutter_animation_set_duration (anim, 750);
  clutter_animation_set_mode (anim, CLUTTER_EASE_IN_QUAD);
  interval = clutter_interval_new (G_TYPE_FLOAT,
                                   (CLUTTER_STAGE_WIDTH () -
                                    clutter_actor_get_width (actor)) / 2.f,
                                   (gfloat)CLUTTER_STAGE_WIDTH ());
  clutter_animation_bind_interval (anim, "x", interval);
  timeline = clutter_animation_get_timeline (anim);
  clutter_timeline_set_delay (timeline, 1300);
  clutter_timeline_start (timeline);
  g_signal_connect (anim, "completed", G_CALLBACK (g_object_unref), NULL);

  return TRUE;
}

int
main (int argc, char **argv)
{
  gfloat width, height;
  ClutterActor *stage, *background, *actor;

  /* Initialise clutter */
  clutter_init (&argc, &argv);

  /* Create a fullscreen stage */
  stage = clutter_stage_get_default ();
  clutter_stage_set_fullscreen (CLUTTER_STAGE (stage), TRUE);
  clutter_actor_get_size (stage, &width, &height);

  /* Add a backgroud at the size of the stage */
  background = clutter_texture_new_from_file ("images/background.jpg", NULL);
  clutter_actor_set_size (background, width, height);
  clutter_container_add_actor (CLUTTER_CONTAINER (stage), background);

  /* Add an actor */
  actor = clutter_texture_new_from_file ("images/actor.png", NULL);
  clutter_actor_set_size (actor, width/5, height/5);
  clutter_actor_hide (actor);
  clutter_container_add_actor (CLUTTER_CONTAINER (stage), actor);

  /* Start animation when we click */
  g_signal_connect_swapped (stage, "button-press-event",
                            G_CALLBACK (animate_cb), actor);

  /* Close when we press a key */
  g_signal_connect (stage, "key-press-event",
                    G_CALLBACK (clutter_main_quit), NULL);

  /* Startup the application */
  clutter_actor_show (stage);
  clutter_main ();

  return 0;
}

