#include <clutter/clutter.h>
#include <nbtk/nbtk.h>

static NbtkWidget *last_button = NULL;
static const gchar *string =
 "Lorem ipsum dolor sit amet, consectetur adipisicing elit, "
 "sed do eiusmod tempor incididunt ut labore et dolore magna "
 "aliqua. Ut enim ad minim veniam, quis nostrud exercitation "
 "ullamco laboris nisi ut aliquip ex ea commodo consequat. "
 "Duis aute irure dolor in reprehenderit in voluptate velit "
 "esse cillum dolore eu fugiat nulla pariatur. Excepteur sint "
 "occaecat cupidatat non proident, sunt in culpa qui officia "
 "deserunt mollit anim id est laborum.";

static void
clicked_cb (NbtkWidget *button)
{
  if (last_button)
    {
      clutter_actor_animate (CLUTTER_ACTOR (last_button),
                             CLUTTER_EASE_IN_OUT_QUAD,
                             750,
                             "width", 0.f,
                             "height", 0.f,
                             NULL);
      if (last_button == button)
        {
          last_button = NULL;
          return;
        }
    }

  clutter_actor_animate (CLUTTER_ACTOR (button),
                         CLUTTER_EASE_IN_OUT_QUAD,
                         750,
                         "width", (gfloat)(CLUTTER_STAGE_WIDTH () - 500),
                         "height", (gfloat)(CLUTTER_STAGE_HEIGHT () - 300),
                         NULL);
  last_button = button;
}

static void
add_button (NbtkWidget *table, gint row, gint col)
{
  NbtkWidget *button = nbtk_button_new_with_label (string);
  ClutterActor *text = nbtk_bin_get_child (NBTK_BIN (button));

  clutter_actor_set_size (CLUTTER_ACTOR (button), 0, 0);

  g_object_set (G_OBJECT (text),
                "single-line-mode", FALSE,
                "line-wrap", TRUE,
                "line-wrap-mode", PANGO_WRAP_WORD_CHAR,
                "ellipsize", PANGO_ELLIPSIZE_END,
                NULL);

  g_signal_connect (button, "clicked", G_CALLBACK (clicked_cb), NULL);

  nbtk_table_add_actor_with_properties (NBTK_TABLE (table),
                                        CLUTTER_ACTOR (button),
                                        row, col,
                                        "x-fill", TRUE,
                                        "y-fill", TRUE,
                                        NULL);
}

int
main (int argc, char **argv)
{
  ClutterActor *stage;
  NbtkWidget *table;

  clutter_init (&argc, &argv);
  nbtk_style_load_from_file (nbtk_style_get_default (),
                             "layout-anim.css",
                             NULL);

  stage = clutter_stage_get_default ();
  clutter_stage_set_fullscreen (CLUTTER_STAGE (stage), TRUE);

  table = nbtk_table_new ();
  clutter_actor_set_size (CLUTTER_ACTOR (table),
                          CLUTTER_STAGE_WIDTH () - 400,
                          CLUTTER_STAGE_HEIGHT () - 200);
  clutter_actor_set_position (CLUTTER_ACTOR (table), 200, 100);

  add_button (table, 0, 0);
  add_button (table, 0, 1);
  add_button (table, 1, 0);
  add_button (table, 1, 1);

  clutter_container_add_actor (CLUTTER_CONTAINER (stage),
                               CLUTTER_ACTOR (table));

  clutter_actor_show (stage);

  clutter_main ();

  return 0;
}
